import numpy as np
from .constants import Constants

class Eos():
    """Equation of state for freshwater from Chen and Millero (1986).

    Attributes
    ----------
    t : array_like
        temperature (:math:`\mathrm{^{\circ} C}`)
    s : array_like
        salinity (g/kg) grams of dissolved salt in 1 kg of lake water
    p : array_like
        pressure (bar) applied pressure
    rho : array_like
        density of lake water (:math:`\mathrm{kg \, m^{-3}}`)
    alpha : array_like
        Coefficient of thermal expansion :math:`\mathrm{(^{\circ} C)^{-1}}`
    tmd : array_like
        Temperature of maximum density (:math:`\mathrm{^{\circ} C}`)
    u : array_like
        Speed of sound (:math:`\mathrm{m \, s^{-1}}`)
    cp : array_like
        Specific heat capacity (:math:`\mathrm{J \, kg^{-1} \, ^{\circ} C^{-1}}`)
    tf : array_like
        Freezing point of lake water (:math:`\mathrm{^{\circ} C}`)

    References
    ----------
    .. [1] Chen, C. T. A., & Millero, F. J. (1986). Thermodynamic properties 
        for natural waters covering only the limnological range 1. Limnology and
        Oceanography, 31(3), 657-662.

    Examples
    --------
    Import the class Eos (Equation of state) from the freshwater module

    >>> from freshwater.eos import Eos

    Compute properties of lake water with a temperature of 10 deg C,
    a salinity of 0.5 g/kg and a pressure of 0 bar (gauge pressure).

    >>> e = Eos(t=10, s=0.5, p=0)
    >>> e.rho
    1000.091963
    >>> e.alpha
    8.9433231e-05
    >>> e.tmd
    3.87295
    """

    def __init__(self, t, s=0, p=0):
        self.t = t
        self.s = s
        self.p = p
        self.c = Constants()
        self._compute()

    def _compute_rho(self):
        c = self.c
        rho = polyval_r(c.a, self.t) + polyval_r(c.b, self.t)*self.s
        if np.any(self.p) != 0:
            K = (polyval_r(c.c, self.t)
                 + polyval_r(c.d, self.t) * self.p
                 + (c.e[0] + c.e[1]*self.t + c.e[2]*self.p) * self.s)
            rho = rho / (1 - self.p/K)
        self.rho = rho * 1000.

    def _compute_alpha(self):
        c = self.c
        self.alpha = ((polyval_r(c.f, self.t)
                      + polyval_r(c.g, self.t) * self.s
                      + (polyval_r(c.h[:3], self.t) + c.h[3]*self.s) * self.p)
                      * 1e-6)

    def _compute_tmd(self):
        c = self.c
        self.tmd = (c.i[0] + c.i[1]*self.p + c.i[2]*self.p**2
                    - (c.i[3]+c.i[4]*self.p)*self.s)

    def _compute_u(self):
        c = self.c
        self.u = (polyval_r(c.j, self.t)
                  + polyval_r(c.k, self.t) * self.s
                  + ((polyval_r(c.l[:3], self.t) + c.l[3] * self.s)
                     * self.p)
                  + c.m[0]*self.p**2)

    def _compute_cp(self):
        c = self.c
        self.cp = (polyval_r(c.n, self.t)
                            + polyval_r(c.o, self.t) * self.s
                            + ((polyval_r(c.p[:3], self.t) + c.p[3] * self.s)
                               * self.p)
                            + c.q[0]*self.p**2) * 1000.

    def _compute_tf(self):
        c = self.c
        self.tf = c.r[0] + c.r[1]*self.s + c.r[2]*self.p

    def _compute(self):
        self._compute_rho()
        self._compute_alpha()
        self._compute_tmd()
        self._compute_u()
        self._compute_cp()
        self._compute_tf()

def polyval_r(p, x):
    """Identical to numpy.polyval except p is an array of coefficients 
       of increasing order instead of decreasing order.
    """
    return np.polyval(np.flipud(p), x)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['freshwater']

package_data = \
{'': ['*']}

install_requires = \
['numpy>=1.19.1,<2.0.0']

setup_kwargs = {
    'name': 'freshwater',
    'version': '0.1.5',
    'description': 'Python module for estimating thermodynamic properties of water in freshwater lakes.',
    'long_description': 'Freshwater Toolbox \n===================\n*Compute thermodynamic properties of lake water*\n\n|Build Status|\n\nThe freshwater toolbox is a python package for estimating thermodynamic properties of water in freshwater bodies. Given temperature, salinity, and pressure, the freshwater toolbox enables the user to estimate the density, thermal expansion coefficient, specific heat capacity, speed of sound, temperature of maximum density, and freezing point of a parcel of lake water. The equation of state described in Chen and Millero (1986) [1]_ is implemented, which is suited for lake waters over the range of temperature, salinity, and pressure of 0--30 °C, 0--0.6 g/kg, and 0--180 bar, respectively.\n\nLinks\n-----\n\n-  Documentation: https://danielrobb.github.io/freshwater\n-  Source code: https://github.com/danielrobb/freshwater\n\nInstallation\n------------\n\nThe easiest way to install the freshwater toolbox is using pip::\n\n    pip install freshwater\n\nYou can also clone the source code from the `github repository <https://github.com/danielrobb/freshwater>`_ and install it with the following commands::\n\n    git clone https://github.com/danielrobb/freshwater\n    cd freshwater/\n    pip install .\n\nQuick Start\n------------\n\nSay you have a sample of lake water with temperature 10 °C, salinity 0.5 g/kg, and pressure 0 bar, and you would like to compute its density 𝜌 and thermal expansion coefficient 𝛼. This can be done with the following commands.\n\nFirst, import the equation of state (Eos) class::\n\n   >>> from freshwater.eos import Eos\n\nSecond, ``Eos`` takes arguments of temperature, salinity and pressure::\n\n   >>> e = Eos(t=10, s=0.5, p=0)\n   >>> e.rho\n   1000.091963\n   >>> e.alpha\n   8.94332e-05\n\nPlease consult the `online documentation <https://danielrobb.github.io/freshwater>`_ for more details.\n\n.. |Build Status| image:: https://travis-ci.org/danielrobb/freshwater.svg?branch=master\n   :target: https://travis-ci.org/github/danielrobb/freshwater\n   :alt: travis-ci build status\n\n.. [1] Chen, C. T. A., & Millero, F. J. (1986). Thermodynamic properties for natural waters covering only the limnological range. *Limnology and Oceanography*, 31(3), 657-662. `doi  <https://doi.org/10.4319/lo.1986.31.3.0657>`_\n\n\n',
    'author': 'danielrobb',
    'author_email': 'danielrobb@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/danielrobb/freshwater',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
